/*
 *  ButtonInterrupt.cpp
 *
 *  Kastara Electronics Embedded Development
 *  Created on: 2023. 4. 3
 */

#include "ButtonInterrupt.h"

namespace AutoLight {
    ButtonInterrupt::ButtonInterrupt(bool _enable_unit_test)
            : pin_interrupt_(0),
              interrupt_timer_(0),
              debounce_time(250),
              is_unit_test_(_enable_unit_test),
              volatile_data_{
                      .num_of_pressed_ = 0,
                      .is_pressed_ = false,
                      .is_changed_ = false,
              } {
    }

    ButtonInterrupt::~ButtonInterrupt() {
        free(this);
    }

    void ButtonInterrupt::attach(uint8_t _pin, void (*_callback)(), uint32_t _timer) {
        pin_interrupt_ = _pin;
        isr_callback_ = _callback;
        interrupt_timer_ = _timer;

        pinMode(pin_interrupt_, INPUT_PULLUP);
        attachInterrupt(pin_interrupt_, isr_callback_, RISING);
    }

    void ButtonInterrupt::readInterrupt(void (*_callback)()) {
        if (millis() - interrupt_timer_ >= debounce_time) {
            volatile_data_.num_of_pressed_++;
            volatile_data_.is_pressed_ = true;
            if (_callback != nullptr) _callback();
            interrupt_timer_ = millis();
        }
    }

    void ButtonInterrupt::resetButton() {
        volatile_data_.is_pressed_ = false;
    }

    volatile bool ButtonInterrupt::isPressed() {
        return volatile_data_.is_pressed_ || volatile_data_.is_changed_;
    }

    volatile int ButtonInterrupt::getNumOfPressed() {
        return volatile_data_.num_of_pressed_;
    }

    int ButtonInterrupt::getPins() {
        return pin_interrupt_;
    }

    ButtonInterrupt *ButtonInterrupt::operator()() {
        return this;
    }

    void ButtonInterrupt::debug() {
        if (!is_unit_test_) return;
        Serial.println("==============================");
        Serial.println("| START DEBUG BUTTON INTERRUPT");
        Serial.println("==============================");

        Serial.print("| pin_interrupt_               : ");
        Serial.print("| ");
        Serial.print(pin_interrupt_);
        Serial.println();

        Serial.print("| num_of_pressed_              : ");
        Serial.print("| ");
        Serial.print(volatile_data_.num_of_pressed_);
        Serial.println();

        Serial.print("| interrupt_timer_             : ");
        Serial.print("| ");
        Serial.print(interrupt_timer_);
        Serial.println();

        Serial.print("| isr_callback_                : ");
        Serial.print("| ");
        Serial.print((isr_callback_ != nullptr) ? "true" : "false");
        Serial.println();

        Serial.println("| END DEBUG BUTTON INTERRUPT");
        Serial.println("==============================");
    }
}